import {} from 'd3';

const capitalizeText = (text) => {
  if (text)
    return text[0].toUpperCase() + text.substring(1);

  return '';
};

export const legend = (
  selection,
  {
    colorScale,
    legendLabel,
    legendX,
    legendY,
    tickSpacing = 30,
    tickPadding = 20,
    hoveredValue,
    setHoveredValue,
  },
) => {
  const colorLegendG = selection
    .selectAll('g.legend')
    .data([null])
    .join('g')
    .attr('class', 'legend')
    .attr('transform', `translate(${legendX}, ${legendY})`);

  colorLegendG
    .selectAll('text.legend-label')
    .data([null])
    .join('text')
    .attr('x', 0)
    .attr('y', -28)
    .attr('class', 'legend-label')
    .attr('font-family', 'sans-serif')
    .attr('font-size', 20)
    .attr('font-weight', 'bold')
    .attr('text-decoration', 'underline')
    .text(legendLabel);

  colorLegendG
    .selectAll('g.tick')
    .data(colorScale.domain())
    .join(
      (enter) =>
        enter
          .append('g')
          .attr('class', 'tick')
          .call((selection) => {
            selection.append('circle');
            selection.append('text');
          }),
      (update) => update,
      (exit) => exit.remove(),
    )
    .attr(
      'transform',
      (d, i) => `translate(0, ${i * tickSpacing})`,
    )
    .attr('font-size', 20)
    .attr('font-family', 'sans-serif')
    .call((selection) => {
      selection
        .selectAll('circle')
        .attr('r', 10)
        .attr('fill', colorScale);
      selection
        .selectAll('text')
        .text((d) => d)
        .style('text-transform', 'capitalize')
        .attr('x', tickPadding)
        .style('user-select', 'none')
        .attr('dy', '0.32em'); // some sort of magic number -> 0.32em=
    })
    .style('cursor', 'pointer')
    .attr('opacity', (d) =>
      hoveredValue ? (d === hoveredValue ? 1 : 0.2) : 1,
    )
    .on('click', (event, d) => {
      hoveredValue
        ? setHoveredValue(null)
        : setHoveredValue(d);
    });
};
