import {
  scaleLinear,
  scaleOrdinal,
  extent,
  schemeCategory10,
} from 'd3';
import { axes } from './axes';
import { legend } from './legend';

export const scatterPlot = (selection, options) => {
  const {
    data,
    width,
    height,
    xValue,
    yValue,
    colorValue,
    margin,
    xAxisLabel,
    yAxisLabel,
    legendLabel,
    legendX,
    legendY,
    hoveredValue,
    setHoveredValue,
  } = options;

  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([margin.left, width - margin.right]);

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([height - margin.bottom, margin.top]);

  const colorScale = scaleOrdinal()
    .domain(data.map(colorValue))
    .range(schemeCategory10);

  const marks = data.map((d) => ({
    x: xScale(xValue(d)),
    y: yScale(yValue(d)),
    color: colorScale(colorValue(d)),
    species: d.species,
  }));

  selection.call(axes, {
    xScale,
    yScale,
    xAxisLabel,
    yAxisLabel,
  });

  selection.call(legend, {
    colorScale,
    legendLabel,
    legendX,
    legendY,
    setHoveredValue,
    hoveredValue,
  });

  // Render a Scatter Plot
  selection
    .selectAll('circle.mark')
    .data(marks)
    .join('circle')
    .attr('class', 'mark')
    .attr('cx', (d) => d.x)
    .attr('cy', (d) => d.y)
    .attr('r', 5)
    .attr('fill', (d) => d.color)
    .attr('opacity', (d) =>
      hoveredValue
        ? d.species === hoveredValue
          ? 1
          : 0.2
        : 1,
    );
};
